@@ -1,6 +1,6 @@
 /*
  *	Miscellaneous functions
- *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 1999 - 2009
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 1999 - 2010
  *
  *	This file is part of libHX. libHX is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU
@@ -34,6 +34,22 @@ EXPORT_SYMBOL int HX_fls(unsigned long n)
 	return -1;
 }
 
+static inline void hexdump_ascii(FILE *fp, unsigned char c, bool tty)
+{
+	static const unsigned char ct_char[] = "31", up_char[] = "34";
+
+	if (HX_isprint(c))
+		fprintf(fp, "%c", c);
+	else if (tty && c == 0)
+		fprintf(fp, "\e[%sm@\e[0m", up_char); // ]]
+	else if (tty && c < 32)
+		fprintf(fp, "\e[%sm%c\e[0m", ct_char, '@' + c); // ]]
+	else if (tty)
+		fprintf(fp, "\e[%sm.\e[0m", up_char); // ]]
+	else
+		fprintf(fp, ".");
+}
+
 EXPORT_SYMBOL void HX_hexdump(FILE *fp, const void *vptr, unsigned int len)
 {
 	const unsigned char *ptr = vptr;
@@ -47,13 +63,8 @@ EXPORT_SYMBOL void HX_hexdump(FILE *fp, const void *vptr, unsigned int len)
 			fprintf(fp, "%02x%c", *ptr++, (j == 7) ? '-' : ' ');
 		ptr -= 16;
 		fprintf(fp, "| ");
-		for (j = 0; j < 16; ++j, ++ptr)
-			if (HX_isprint(*ptr))
-				fprintf(fp, "%c", *ptr);
-			else if (tty)
-				fprintf(fp, "\e[31m.\e[0m"); // ]]
-			else
-				fprintf(fp, ".");
+		for (j = 0; j < 16; ++j)
+			hexdump_ascii(fp, *ptr++, tty);
 		fprintf(fp, "\n");
 	}
 	fprintf(fp, "%04x | ", i * 16);
@@ -64,12 +75,7 @@ EXPORT_SYMBOL void HX_hexdump(FILE *fp, const void *vptr, unsigned int len)
 		fprintf(fp, "   ");
 	fprintf(fp, "| ");
 	for (i = 0; i < len; ++i)
-		if (HX_isprint(ptr[i]))
-			fprintf(fp, "%c", ptr[i]);
-		else if (tty)
-			fprintf(fp, "\e[31m.\e[0m"); // ]]
-		else
-			fprintf(fp, ".");
+		hexdump_ascii(fp, *ptr++, tty);
 	fprintf(fp, "\n");
 }
 
