@@ -10,6 +10,7 @@ A=b;C="d" ; E="F;" ; F= G=Z
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <libHX/defs.h>
 #include <libHX/init.h>
 #include <libHX/map.h>
 #include <libHX/option.h>
@@ -17,6 +18,11 @@ A=b;C="d" ; E="F;" ; F= G=Z
 static const char *const fmt2_strings[] = {
 	"HOME=%(env HOME)\n",
 	"USER=%(upper %(lower %(env USER)))\n",
+	"lower: %(lower foo, bar) %(lower %(TWOARG))\n",
+	"substr-1: %(substr Hello World,0,5) %(substr Hello World,-5,5)\n",
+	"substr-2: %(substr Hello World,-15,-6) %(substr Hello World,6)\n",
+	"substr-3: %(substr Hello World,-99) <%(substr Hello World,12,2)>\n",
+	"substr-4: <%(substr Hello World,-22,-11)> %(substr Hello World,-12,5)\n",
 	"no-exp: %%(NOEXPANSION) %NOEXPANSION\n",
 	"empty-1: <%()>\n",
 	"empty-2: <%( )>\n",
@@ -59,9 +65,10 @@ static void t_format(int argc)
 	HXformat_add(fmt, "jengelh", "1337", HXTYPE_STRING | HXFORMAT_IMMED);
 	HXformat_add(fmt, "USER", "jengelh", HXTYPE_STRING | HXFORMAT_IMMED);
 	HXformat_add(fmt, "ARGC", &argc, HXTYPE_INT);
-	HXformat_add(fmt, "ARGK", (const void *)(long)argc, HXTYPE_INT | HXFORMAT_IMMED);
+	HXformat_add(fmt, "ARGK", reinterpret_cast(const void *, static_cast(long, argc)), HXTYPE_INT | HXFORMAT_IMMED);
 	HXformat_add(fmt, "ZERO", "", HXTYPE_STRING | HXFORMAT_IMMED);
 	HXformat_add(fmt, "ONE", "1", HXTYPE_STRING | HXFORMAT_IMMED);
+	HXformat_add(fmt, "TWOARG", "a, b", HXTYPE_STRING | HXFORMAT_IMMED);
 	++argc;
 	printf("# HXformat2\n");
 	for (s = fmt2_strings; *s != '\0'; ++s)
@@ -149,12 +156,13 @@ int main(int argc, const char **argv)
 {
 	if (HX_init() <= 0)
 		abort();
+	printf("Return value of HX_getopt: %d\n",
+	       HX_getopt(table, &argc, &argv, HXOPT_USAGEONERR));
+
 	t_format(argc);
 	t_shconfig((argc >= 2) ? argv[1] : "tc-option.c");
 	t_shconfig2((argc >= 2) ? argv[1] : "tc-option.c");
 
-	printf("Return value of HX_getopt: %d\n",
-	       HX_getopt(table, &argc, &argv, HXOPT_USAGEONERR));
 	printf("Either-or is: %s\n", opt_eitheror[opt_dst]);
 	printf("values: D=%lf I=%d L=%ld S=%s\n",
 	       opt_kdbl, opt_kint, opt_klong, opt_kstr);
